%% DEMO_SOMDDPA
% This is a demo script for the second-order modally damped dominant pole
% algorithm. It demonstrates the usage of the somddpa function and the
% different optional parameters on very simple constructed modally-damped
% second-order system.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2019 Peter Benner, Steffen W. R. Werner
%


%% Construct modally damped second-order system.
n = 100; % States.
m = 2;  % Inputs.
p = 3;  % Outputs.

if exist('OCTAVE_VERSION', 'builtin')
    % Build example in Octave.
    rand('seed', 1.0);
    U = randn(n);
    
    M = U' * diag(1:n) * U;
    E = U' * diag([1/(2 * n) * (1:n/2), 2*n+1:2*n+n/2]) * U;
    K = U' * diag(n+1:2*n) * U;
    
    rand('seed', 2.0);
    B = rand(n, m);
    rand('seed', 3.0);
    C = rand(p, n);
else
    % Build example in MATLAB.
    rng(1.0);
    U = randn(n);
    
    M = U' * diag(1:n) * U;
    E = U' * diag([1/(2 * n) * (1:n/2), 2*n+1:2*n+n/2]) * U;
    K = U' * diag(n+1:2*n) * U;
    
    rng(2.0);
    B = rand(n, m);
    rng(3.0);
    C = rand(p, n);
end


%% Define options for the algorithm.
opts = struct( ...
    'DPStrategy'   , 'LRP', ...
    'Expansion'    , 'full_qr', ...
    'Kmin'         , 6, ...
    'Kmax'         , 30, ...
    'Linearization', 3, ...
    'MaxIter'      , 100, ...
    'Nwanted'      , 6, ...
    'Output'       , 2, ...
    'Reortho'      , 1, ...
    'Target'       , [], ...
    'TDefl'        , 1, ...
    'Tolerance'    , 1.0e-08);


%% Initial guess of dominant pole pair.
s0 = -1.0;


%% Call the function.
[lambda_p, lambda_m, X, info] = somddpa(M, E, K, B, C, s0, opts);
